;Rock and Scroll
;by Andrew Hunt
;from Your Sinclair, Feb'92

;[There are a couple of bugs in this original version.                ]
;[The subroutine at 65131 copies one too many bytes on each iteration,]
;[causing the left-most column to get copied to the right-most column ]
;[before the next column is scrolled in.                              ]
;[The subroutine at 65180 copies one too many bytes when resetting the]
;[attributes, overwriting the first byte of the printer buffer and so ]
;[causing a crash if run on the 128K.                                 ]
;[I've done a corrected version in ROCKSCRL.TAP "RockSfix.o".     JimG]

65000 54       ;1st char of message (held backwards)
65001 0A00     ;offset within message
65003 00000000 ;}current character's
65007 00181800 ;}definition
65011 0A00     ;length of message
65013 07       ;colour
65014 003C7E7E ;}UDG
65018 7E7E3C00 ;}definition

;start with reset
65022 CD9CFE   CALL 65180       ;reset attributes & copy UDG across whole screen
;start without reset
65025 ED4BF3FD LD   BC,(65011)  ;length of message
65029 C5       PUSH BC          
65030 ED5BE9FD LD   DE,(65001)  ;offset within message
65034 D5       PUSH DE          
65035 CD1EFE   CALL 65054       ;get current char definition
65038 CD7CFE   CALL 65148       ;display current character
65041 D1       POP  DE          
65042 13       INC  DE          ;increment offset within message
65043 ED53E9FD LD   (65001),DE  ;save
65047 C1       POP  BC          
65048 0B       DEC  BC          ;decrement remaining message length
65049 78       LD   A,B         
65050 B1       OR   C           
65051 20E8     JR   NZ,65029    ;loop back if more to do
65053 C9       RET              

;copy current char definition into workspace
65054 21E8FD   LD   HL,65000    ;start of message
65057 ED52     SBC  HL,DE       ;subtract offset
65059 6E       LD   L,(HL)      ;get char code to display
65060 7D       LD   A,L         
65061 D620     SUB  032         ;" "=0, "!"=1, etc.
65063 6F       LD   L,A         
65064 2600     LD   H,000       
65066 29       ADD  HL,HL       
65067 29       ADD  HL,HL       
65068 29       ADD  HL,HL       
65069 ED5B365C LD   DE,(23606)  ;(CHARS)
65073 19       ADD  HL,DE       
65074 24       INC  H           
65075 11EBFD   LD   DE,65003    
65078 010800   LD   BC,00008    
65081 EDB0     LDIR             ;copy char definition into workspace
65083 C9       RET              

;set attributes for current column
65084 AF       XOR  A           
65085 0608     LD   B,008       
65087 112000   LD   DE,00032    
65090 211F58   LD   HL,22559    ;(start of attributes)+31
65093 DD21EBFD LD   IX,65003    ;start of current character's definition
65097 DDCB007E BIT  7,(IX+000)  
65101 2805     JR   Z,65108     
65103 3AF5FD   LD   A,(65013)   ;set colour as specified if bit 7 set
65106 1801     JR   65109       
65108 AF       XOR  A           ;set colour as black if bit 7 reset
65109 77       LD   (HL),A      
65110 19       ADD  HL,DE       ;+32 (same column, next row)
65111 77       LD   (HL),A      
65112 19       ADD  HL,DE       ;+32
65113 77       LD   (HL),A      
65114 19       ADD  HL,DE       ;+32
65115 DD23     INC  IX          
65117 10EA     DJNZ 65097       ;repeat for 3*8=24 rows of this column
65119 C9       RET              

;rotate left all 8 bytes of current char definition
65120 0608     LD   B,008       
65122 21EBFD   LD   HL,65003    
65125 CB06     RLC  (HL)        
65127 23       INC  HL          
65128 10FB     DJNZ 65125       
65130 C9       RET              

;move down whole 768 bytes of attributes by one byte in 32 byte chunks
65131 210158   LD   HL,22529    
65134 110058   LD   DE,22528    
65137 3E18     LD   A,024       
65139 012000   LD   BC,00032    
65142 EDB0     LDIR             
65144 3D       DEC  A           
65145 20F8     JR   NZ,65139    
65147 C9       RET              

;display current character
65148 0608     LD   B,008       
65150 C5       PUSH BC          
65151 CD3CFE   CALL 65084       
65154 CD6BFE   CALL 65131       
65157 76       HALT             
65158 CD3CFE   CALL 65084       
65161 CD6BFE   CALL 65131       
65164 76       HALT             
65165 CD60FE   CALL 65120       ;rotate left all 8 bytes of current char def
65168 010001   LD   BC,00256    ;}
65171 0B       DEC  BC          ;}
65172 78       LD   A,B         ;}delay loop
65173 B1       OR   C           ;}
65174 20FB     JR   NZ,65171    ;}
65176 C1       POP  BC          
65177 10E3     DJNZ 65150       ;repeat for all 8 bytes of current char def
65179 C9       RET              

;reset attributes & copy UDG across whole screen
65180 210058   LD   HL,22528    
65183 110158   LD   DE,22529    
65186 010003   LD   BC,00768    
65189 75       LD   (HL),L      
65190 EDB0     LDIR             ;reset attributes to black INK & PAPER
65192 210040   LD   HL,16384    
65195 CDB1FE   CALL 65201       ;copy UDG into 1st screen third
65198 CDB1FE   CALL 65201       ;copy UDG into 2nd screen third
                                ;copy UDG into 3rd screen third
65201 11F6FD   LD   DE,65014    ;location of UDG definition
65204 0E08     LD   C,008       ;8 bytes
65206 06FF     LD   B,255       
65208 1A       LD   A,(DE)      
65209 77       LD   (HL),A      
65210 23       INC  HL          
65211 10FC     DJNZ 65209       
65213 77       LD   (HL),A      
65214 23       INC  HL          
65215 13       INC  DE          
65216 0D       DEC  C           
65217 20F3     JR   NZ,65206    
65219 C9       RET              
